/**
 * @file
 * @brief This tests parsing and encoding input based on different protocols and inputs
 *
 * Parameters to test:
 * protocol
 * valid and invalid input
 * 
 * Compare actual output with expected output
 *
 * Netsweeper Squid URL Redirector
 * This application is designed to work with the Netsweeper Internet Content Filtering System.
 *
 * Copyright 2010 Netsweeper Inc. All Rights Reserved.
 * Netsweeper, Netsweeper Inc. is either a copyrighted or registered trademark of Netsweeper Inc.
 */
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <pthread.h>

#include "CUnit/Basic.h"

#include "protocol.h"

static void protocol_parse_valid_squid25_test()
{
	protocol_select squid25 = SQUID_25;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;

	GlobalConf conf;
	conf.server_concurrency = 0;
	
	// parse
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method\n", &parsed_line, squid25)) {
		CU_FAIL("Valid input parsing failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	
	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid25);


	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method\n")) {
		CU_FAIL("Protocol encoding failed for squid25 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}


	CU_PASS("Protocol parse with valid squid25 input test has passed");
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}
	free(line_buffer);
	return;
}

static void protocol_parse_valid_squid26_test()
{
	protocol_select squid26 = SQUID_26;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	conf.server_concurrency = 0;
	
	// parse
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method group\n", &parsed_line, squid26)) {
		CU_FAIL("Valid input parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->group, "group")) {
		CU_FAIL("Group parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}


	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid26);


	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method group\n")) {
		CU_FAIL("Protocol encoding failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid squid26 input test has passed");
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}
	free(line_buffer);
	return;
}

static void protocol_parse_valid_concurrent_squid26_test()
{
	protocol_select squid26 = SQUID_26;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	
	// parse
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method group\n", &parsed_line, squid26)) {
		CU_FAIL("Valid concurrent input parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	
	if(strcmp(parsed_line->req_id, "1024")) {
		CU_FAIL("Request id parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->group, "group")) {
		CU_FAIL("Group parsing failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}

	conf.server_concurrency = 0;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid26);
	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method group\n")) {
		CU_FAIL("Protocol encoding failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}
	conf.server_concurrency = 1;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid26);
	if(strcmp(line_buffer, "1024 http://www.netsweeper.com ip/fqdn ident method group\n")) {
		CU_FAIL("Protocol encoding failed for squid26 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid squid26 concurrent input test has passed");
	protocol_parsed_line_destroy(parsed_line);
	free(line_buffer);
	return;
}

static void protocol_parse_valid_squid27_test()
{
	protocol_select squid27 = SQUID_27;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	conf.server_concurrency = 0;
	
	// parse
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method group key1=value1 key2=value2\n", &parsed_line, squid27)) {
		CU_FAIL("Valid input parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->group, "group")) {
		CU_FAIL("Group parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->otheroptions[0], "key1=value1")) {
		CU_FAIL("Key-value pair parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->otheroptions[1], "key2=value2")) {
		CU_FAIL("Key-value pair parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid27);


	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method group key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid squid27 input test has passed");
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}
	free(line_buffer);
	return;
}

static void protocol_parse_valid_concurrent_squid27_test()
{
	protocol_select squid27 = SQUID_27;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	
	// parse
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method group key1=value1 key2=value2\n", &parsed_line, squid27)) {
		CU_FAIL("Valid input parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->req_id, "1024")) {
		CU_FAIL("Request id parsing failed for squid27 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->group, "group")) {
		CU_FAIL("Group parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->otheroptions[0], "key1=value1")) {
		CU_FAIL("Key-value pair parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->otheroptions[1], "key2=value2")) {
		CU_FAIL("Key-value pair parsing failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}

	conf.server_concurrency = 0;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid27);
	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method group key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}
	conf.server_concurrency = 1;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid27);
	if(strcmp(line_buffer, "1024 http://www.netsweeper.com ip/fqdn ident method group key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid27 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid concurrent squid27 input test has passed");
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}
	free(line_buffer);
	return;
}

static void protocol_parse_valid_squid30_test()
{
	protocol_select squid30 = SQUID_30;
	//protocol_select squid30 = SQUID_25;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	conf.server_concurrency = 0;
	
	// parse
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method key1=value1 key2=value2\n", &parsed_line, squid30)) {
		CU_FAIL("Valid input parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	if(strcmp(parsed_line->otheroptions[0], "key1=value1")) {
		CU_FAIL("Key-value pair parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(strcmp(parsed_line->otheroptions[1], "key2=value2")) {
		CU_FAIL("Key-value pair parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		exit(-1);
	}
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid30);


	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid squid30 input test has passed");
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}
	free(line_buffer);
	return;
}

static void protocol_parse_valid_concurrent_squid30_test()
{
	protocol_select squid30 = SQUID_30;
	protocol_input_line *parsed_line = NULL;
	char *line_buffer;
	int line_buffer_len = 1024;
	GlobalConf conf;
	
	// parse
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method key1=value1 key2=value2\n", &parsed_line, squid30)) {
		CU_FAIL("Valid concurrent input parsing failed for squid30 protocol");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	
	if(strcmp(parsed_line->req_id, "1024")) {
		CU_FAIL("Request id parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->url, "http://www.netsweeper.com")) {
		CU_FAIL("URL parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->IP, "ip/fqdn")) {
		CU_FAIL("IP parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->username, "ident")) {
		CU_FAIL("Identity parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->method, "method")) {
		CU_FAIL("Method parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->otheroptions[0], "key1=value1")) {
		CU_FAIL("Key-value pair parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}
	if(strcmp(parsed_line->otheroptions[1], "key2=value2")) {
		CU_FAIL("Key-value pair parsing failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		return;
	}

	// encode
	if((line_buffer = malloc(line_buffer_len + 1)) == NULL) {
		fprintf(stderr, "Failed to allocate memory\n");
		protocol_parsed_line_destroy(parsed_line);
		exit(-1);
	}
	
	conf.server_concurrency = 0;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid30);
	if(strcmp(line_buffer, "http://www.netsweeper.com ip/fqdn ident method key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		free(line_buffer);
		return;
	}
	conf.server_concurrency = 1;
	protocol_encode(&conf, parsed_line, line_buffer, line_buffer_len, squid30);
	if(strcmp(line_buffer, "1024 http://www.netsweeper.com ip/fqdn ident method key1=value1 key2=value2\n")) {
		CU_FAIL("Protocol encoding failed for squid30 protocol");
		protocol_parsed_line_destroy(parsed_line);
		free(line_buffer);
		return;
	}

	CU_PASS("Protocol parse with valid squid30 concurrent input test has passed");
	protocol_parsed_line_destroy(parsed_line);
	free(line_buffer);
	return;
}

static void protocol_parse_invalid_squid25_test()
{
	protocol_input_line *parsed_line = NULL;
	GlobalConf conf;
	conf.server_concurrency = 0;

	// boundary inputs	
	if(protocol_parse(NULL, &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse(" ", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// no enough fields
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident \n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid schema separator
	parsed_line = NULL;
	if(protocol_parse("http//www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// too much fields
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method more more more more\n", &parsed_line, SQUID_25) == 0) {
		CU_FAIL("Protocol parse with invalid squid25 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	CU_PASS("Protocol parse with invalid squid25 input test has passed");
	return;
}

static void protocol_parse_invalid_squid26_test()
{
	protocol_input_line *parsed_line = NULL;
	GlobalConf conf;
	conf.server_concurrency = 0;

	// no enough fields
	if(protocol_parse("http://www.netsweeper.com\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident \n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method \n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// too much fields
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method group more\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method group more\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid req id
	parsed_line = NULL;
	if(protocol_parse("a1024 http://www.netsweeper.com ip/fqdn ident method group\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// boundary inputs	
	parsed_line = NULL;
	if(protocol_parse(NULL, &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line != NULL) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse(" ", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid schema separator
	parsed_line = NULL;
	if(protocol_parse("http//www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_26) == 0) {
		CU_FAIL("Protocol parse with invalid squid26 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	CU_PASS("Protocol parse with invalid squid26 input test has passed");
	return;
}

static void protocol_parse_invalid_squid27_test()
{
	protocol_input_line *parsed_line = NULL;
	GlobalConf conf;
	conf.server_concurrency = 0;

	// boundary inputs
	if(protocol_parse(NULL, &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse(" ", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// no enough fields
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident \n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method \n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid schema separator
	parsed_line = NULL;
	if(protocol_parse("http//www.netsweeper.com ip/fqdn ident method group\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid key-value pair
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method group key\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method group =key\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method group key=\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid request id
	parsed_line = NULL;
	if(protocol_parse("a1024 http://www.netsweeper.com ip/fqdn ident method group key1=value1\n", &parsed_line, SQUID_27) == 0) {
		CU_FAIL("Protocol parse with invalid squid27 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	CU_PASS("Protocol parse with invalid squid27 input test has passed");
	return;
}

static void protocol_parse_invalid_squid30_test()
{
	protocol_input_line *parsed_line = NULL;
	GlobalConf conf;
	conf.server_concurrency = 0;

	// boundary inputs
	if(protocol_parse(NULL, &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse(" ", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// no enough fields
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident \n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid schema separator
	parsed_line = NULL;
	if(protocol_parse("http//www.netsweeper.com ip/fqdn ident method\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid key-value pair
	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method key\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method key=\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("http://www.netsweeper.com ip/fqdn ident method =value\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	parsed_line = NULL;
	if(protocol_parse("1024 http://www.netsweeper.com ip/fqdn ident method key\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	// invalid request id
	parsed_line = NULL;
	if(protocol_parse("a1024 http://www.netsweeper.com ip/fqdn ident method key1=value1\n", &parsed_line, SQUID_30) == 0) {
		CU_FAIL("Protocol parse with invalid squid30 input test has failed");
		if(parsed_line) {
			protocol_parsed_line_destroy(parsed_line);
		}
		return;
	}
	if(parsed_line) {
		protocol_parsed_line_destroy(parsed_line);
	}

	CU_PASS("Protocol parse with invalid squid30 input test has passed");
	return;
}

int protocol_parse_test_init()
{
	CU_pSuite pSuite = NULL;

	// This is our test suite for test4
	pSuite = CU_add_suite("Protocol Parsing Tests", NULL, NULL);
	if (NULL == pSuite) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid squid25 input test\n", protocol_parse_valid_squid25_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid squid26 input test\n", protocol_parse_valid_squid26_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid concurrent squid26 input test\n", protocol_parse_valid_concurrent_squid26_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid squid27 input test\n", protocol_parse_valid_squid27_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid concurrent squid27 input test\n", protocol_parse_valid_concurrent_squid27_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid squid30 input test\n", protocol_parse_valid_squid30_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();
}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - valid concurrent squid30 input test\n", protocol_parse_valid_concurrent_squid30_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - invalid squid25 input test\n", protocol_parse_invalid_squid25_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - invalid squid26 input test\n", protocol_parse_invalid_squid26_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - invalid squid27 input test\n", protocol_parse_invalid_squid27_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	if ( (NULL == CU_add_test(pSuite, "Protocol parse - invalid squid30 input test\n", protocol_parse_invalid_squid30_test)) ) {
		CU_cleanup_registry();
		return CU_get_error();

	}

	return 0;
}
